@@ -20,9 +20,9 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 2507 2007-06-27 19:01:05Z wwalker $"
-__version__   = "$Revision: 2507 $"
-__date__      = "$Date: 2007-06-27 15:01:05 -0400 (Wed, 27 Jun 2007) $"
+__id__        = "$Id: mag.py 2828 2007-09-13 19:26:33Z wwalker $"
+__version__   = "$Revision: 2828 $"
+__date__      = "$Date: 2007-09-13 15:26:33 -0400 (Thu, 13 Sep 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -454,7 +454,7 @@ def magnifyAccessible(event, obj, extents=None):
 
     if extents:
         [x, y, width, height] = extents
-    elif event and (event.type == "object:text-caret-moved") \
+    elif event and event.type.startswith("object:text-caret-moved") \
        and obj.text and (obj.text.caretOffset >= 0):
         offset = obj.text.caretOffset
         [x, y, width, height] = obj.text.getCharacterExtents(offset,
