@@ -19,9 +19,9 @@
 
 """Displays a GUI for the Orca main window."""
 
-__id__        = "$Id: orca_gui_main.py 1515 2006-10-02 19:28:57Z richb $"
-__version__   = "$Revision: 1515 $"
-__date__      = "$Date: 2006-10-02 15:28:57 -0400 (Mon, 02 Oct 2006) $"
+__id__        = "$Id: orca_gui_main.py 2232 2007-04-02 17:17:42Z richb $"
+__version__   = "$Revision: 2232 $"
+__date__      = "$Date: 2007-04-02 13:17:42 -0400 (Mon, 02 Apr 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -82,7 +82,7 @@ class orcaMainGUI(orca_glade.GladeWrapper):
         - widget: the component that generated the signal.
         """
 
-        orca._showQuitGUI()
+        orca.quitOrca()
 
     def preferencesButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the preferencesButton
@@ -107,7 +107,7 @@ class orcaMainGUI(orca_glade.GladeWrapper):
         global OS
 
         OS = None
-        orca._showQuitGUI()
+        orca.quitOrca()
 
 def showMainUI():
     global OS
