@@ -19,9 +19,9 @@
 
 """Custom script for gedit."""
 
-__id__        = "$Id: gedit.py 2371 2007-05-11 15:28:14Z richb $"
-__version__   = "$Revision: 2371 $"
-__date__      = "$Date: 2007-05-11 11:28:14 -0400 (Fri, 11 May 2007) $"
+__id__        = "$Id: gedit.py 2828 2007-09-13 19:26:33Z wwalker $"
+__version__   = "$Revision: 2828 $"
+__date__      = "$Date: 2007-09-13 15:26:33 -0400 (Thu, 13 Sep 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -176,14 +176,13 @@ class Script(default.Script):
             lastEndOffset = -1
             while offset < length:
                 [mystr, start, end] = text.getTextAtOffset(offset, mode)
+                endOffset = end
 
                 if len(mystr) != 0:
                     string += mystr
 
                 if mode == atspi.Accessibility.TEXT_BOUNDARY_LINE_START or \
                    len(mystr) == 0 or mystr[len(mystr)-1] in '.?!':
-                    endOffset = end
-
                     string = self.adjustForRepeats(string)
                     if string.isupper():
                         voice = settings.voices[settings.UPPERCASE_VOICE]
@@ -534,7 +533,7 @@ class Script(default.Script):
         # from the text area when it regains focus.
         # (See bug #350854 for more details).
         #
-        if (event.type == "object:state-changed:focused") \
+        if event.type.startswith("object:state-changed:focused") \
            and (event.detail1):
             orca.setLocusOfFocus(event, event.source)
 
