@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 2750 2007-09-01 19:15:08Z joanied $"
-__version__   = "$Revision: 2750 $"
-__date__      = "$Date: 2007-09-01 15:15:08 -0400 (Sat, 01 Sep 2007) $"
+__id__        = "$Id: speechgenerator.py 2796 2007-09-06 18:12:59Z shaeger $"
+__version__   = "$Revision: 2796 $"
+__date__      = "$Date: 2007-09-06 14:12:59 -0400 (Thu, 06 Sep 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1130,20 +1130,6 @@ class SpeechGenerator:
         else:
             utterances = []
             utterances.extend(self._getSpeechForObjectLabel(obj))
-
-            # Ignore the text on the slider.  See bug 340559
-            # (http://bugzilla.gnome.org/show_bug.cgi?id=340559): the
-            # implementors of the slider support decided to put in a
-            # Unicode left-to-right character as part of the text,
-            # even though that is not painted on the screen.
-            #
-            # In Java, however, there are sliders without a label. In
-            # this case, we'll add to presentation the slider name if
-            # it exists and we haven't found anything yet.
-            #
-            if not utterances:
-                utterances.extend(self._getSpeechForObjectName(obj))
-
             utterances.extend(self._getSpeechForObjectRole(obj))
             utterances.append(valueString)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
@@ -1219,7 +1205,7 @@ class SpeechGenerator:
         hasItems = False
         for i in range(0, obj.childCount):
             child = obj.child(i)
-            if child.state.count(atspi.Accessibility.STATE_SHOWING):
+            if child and child.state.count(atspi.Accessibility.STATE_SHOWING):
                 hasItems = True
                 break
         if not hasItems:
